@@ -96,12 +96,12 @@ atof_generic (
   char digits_sign_char;
 
   /*
-	 * Scan the input string, abstracting (1)digits (2)decimal mark (3) exponent.
-	 * It would be simpler to modify the string, but we don't; just to be nice
-	 * to caller.
-	 * We need to know how many digits we have, so we can allocate space for
-	 * the digits' value.
-	 */
+   * Scan the input string, abstracting (1)digits (2)decimal mark (3) exponent.
+   * It would be simpler to modify the string, but we don't; just to be nice
+   * to caller.
+   * We need to know how many digits we have, so we can allocate space for
+   * the digits' value.
+   */
 
   char *p;
   char c;
@@ -300,13 +300,13 @@ atof_generic (
       int count;		/* Number of useful digits left to scan. */
 
       LITTLENUM_TYPE *digits_binary_low;
-      int precision;
-      int maximum_useful_digits;
-      int number_of_digits_to_use;
-      int more_than_enough_bits_for_digits;
-      int more_than_enough_littlenums_for_digits;
-      int size_of_digits_in_littlenums;
-      int size_of_digits_in_chars;
+      unsigned int precision;
+      unsigned int maximum_useful_digits;
+      unsigned int number_of_digits_to_use;
+      unsigned int more_than_enough_bits_for_digits;
+      unsigned int more_than_enough_littlenums_for_digits;
+      unsigned int size_of_digits_in_littlenums;
+      unsigned int size_of_digits_in_chars;
       FLONUM_TYPE power_of_10_flonum;
       FLONUM_TYPE digits_flonum;
 
@@ -455,8 +455,8 @@ atof_generic (
 	/* TRUE:  decimal_exponent is negative */
 	FLONUM_TYPE temporary_flonum;
 	LITTLENUM_TYPE *temporary_binary_low;
-	int size_of_power_in_littlenums;
-	int size_of_power_in_chars;
+	unsigned int size_of_power_in_littlenums;
+	unsigned int size_of_power_in_chars;
 
 	size_of_power_in_littlenums = precision;
 	/* Precision has a built-in fudge factor so we get a few guard bits. */
