@@ -1,5 +1,5 @@
 /* coff object file format
-   Copyright (C) 1989, 1990, 1991, 1992 Free Software Foundation, Inc.
+   Copyright (C) 1989, 1990, 1991, 1992, 1993 Free Software Foundation, Inc.
 
    This file is part of GAS.
 
@@ -31,11 +31,8 @@ const short seg_N_TYPE[] =
   C_DATA_SECTION,
   C_BSS_SECTION,
   C_UNDEF_SECTION,		/* SEG_UNKNOWN */
-  C_UNDEF_SECTION,		/* SEG_ABSENT */
-  C_UNDEF_SECTION,		/* SEG_PASS1 */
   C_UNDEF_SECTION,		/* SEG_GOOF */
-  C_UNDEF_SECTION,		/* SEG_BIG */
-  C_UNDEF_SECTION,		/* SEG_DIFFERENCE */
+  C_UNDEF_SECTION,		/* SEG_EXPR */
   C_DEBUG_SECTION,		/* SEG_DEBUG */
   C_NTV_SECTION,		/* SEG_NTV */
   C_PTV_SECTION,		/* SEG_PTV */
@@ -74,10 +71,9 @@ static void obj_coff_def PARAMS ((int what));
 static void obj_coff_dim PARAMS ((void));
 static void obj_coff_endef PARAMS ((void));
 static void obj_coff_line PARAMS ((void));
-static void obj_coff_ln PARAMS ((void));
+static void obj_coff_ln PARAMS ((int));
 static void obj_coff_scl PARAMS ((void));
 static void obj_coff_size PARAMS ((void));
-static void obj_coff_stab PARAMS ((int what));
 static void obj_coff_tag PARAMS ((void));
 static void obj_coff_type PARAMS ((void));
 static void obj_coff_val PARAMS ((void));
@@ -102,6 +98,7 @@ const pseudo_typeS obj_pseudo_table[] =
   {"endef", obj_coff_endef, 0},
   {"line", obj_coff_line, 0},
   {"ln", obj_coff_ln, 0},
+  {"appline", obj_coff_ln, 1},
   {"scl", obj_coff_scl, 0},
   {"size", obj_coff_size, 0},
   {"tag", obj_coff_tag, 0},
@@ -122,16 +119,6 @@ const pseudo_typeS obj_pseudo_table[] =
 
   {"ident", s_ignore, 0},	/* we don't yet handle this. */
 
-
-  /* stabs aka a.out aka b.out directives for debug symbols.
-     Currently ignored silently.  Except for .line at which
-     we guess from context. */
-  {"desc", s_ignore, 0},	/* def */
-  {"stabd", obj_coff_stab, 'd'},/* stabs */
-  {"stabn", obj_coff_stab, 'n'},/* stabs */
-  {"stabs", obj_coff_stab, 's'},/* stabs */
-
-  /* stabs-in-coff (?) debug pseudos (ignored) */
   {"optim", s_ignore, 0},	/* For sun386i cc (?) */
   /* other stuff */
   {"ABORT", s_abort, 0},
@@ -480,6 +467,9 @@ obj_symbol_to_chars (where, symbolP)
      char **where;
      symbolS *symbolP;
 {
+  /* Move the value into the COFF symbol itself.  */
+  symbolP->sy_symbol.ost_entry.n_value = S_GET_VALUE (symbolP);
+
 #ifdef BFD_HEADERS
   unsigned int numaux = symbolP->sy_symbol.ost_entry.n_numaux;
   unsigned int i;
@@ -1141,10 +1131,12 @@ add_linesym (sym)
 #endif
 
 static void
-obj_coff_ln ()
+obj_coff_ln (appline)
+     int appline;
 {
   int l;
-  if (def_symbol_in_progress != NULL)
+
+  if (! appline && def_symbol_in_progress != NULL)
     {
       as_warn (".ln pseudo-op inside .def/.endef: ignored.");
       demand_empty_rest_of_line ();
@@ -1164,7 +1156,9 @@ obj_coff_ln ()
 
     if (listing)
       {
-	listing_source_line (l + line_base - 1);
+	if (! appline)
+	  l += line_base - 1;
+	listing_source_line (l);
       }
   }
 #endif
@@ -1233,6 +1227,7 @@ obj_coff_def (what)
 #endif
 
   def_symbol_in_progress->sy_frag = &zero_address_frag;
+  S_SET_VALUE (def_symbol_in_progress, 0);
 
   if (S_IS_STRING (def_symbol_in_progress))
     SF_SET_STRING (def_symbol_in_progress);
@@ -1486,7 +1481,7 @@ obj_coff_line ()
 
   if (def_symbol_in_progress == NULL)
     {
-      obj_coff_ln ();
+      obj_coff_ln (0);
       return;
     }				/* if it looks like a stabs style line */
 
@@ -1612,11 +1607,15 @@ obj_coff_val ()
 	}
       else if (strcmp (S_GET_NAME (def_symbol_in_progress), symbol_name))
 	{
-	  def_symbol_in_progress->sy_forward = symbol_find_or_make (symbol_name);
+	  def_symbol_in_progress->sy_value.X_op = O_symbol;
+	  def_symbol_in_progress->sy_value.X_add_symbol =
+	    symbol_find_or_make (symbol_name);
+	  def_symbol_in_progress->sy_value.X_op_symbol = NULL;
+	  def_symbol_in_progress->sy_value.X_add_number = 0;
 
-	  /* If the segment is undefined when the forward
-			   reference is solved, then copy the segment id
-			   from the forward symbol. */
+	  /* If the segment is undefined when the forward reference is
+	     resolved, then copy the segment id from the forward
+	     symbol.  */
 	  SF_SET_GET_SEGMENT (def_symbol_in_progress);
 	}
       /* Otherwise, it is the name of a non debug symbol and its value will be calculated later. */
@@ -1649,7 +1648,7 @@ tag_insert (name, symbolP)
 {
   register char *error_string;
 
-  if (*(error_string = hash_jam (tag_hash, name, (char *) symbolP)))
+  if ((error_string = hash_jam (tag_hash, name, (char *) symbolP)))
     {
       as_fatal ("Inserting \"%s\" into structure table failed: %s",
 		name, error_string);
@@ -1712,32 +1711,6 @@ obj_crawl_symbol_chain (headers)
   /* Initialize the stack used to keep track of the matching .bb .be */
   stack *block_stack = stack_init (512, sizeof (symbolS *));
 
-  /* JF deal with forward references first... */
-  for (symbolP = symbol_rootP; symbolP; symbolP = symbol_next (symbolP))
-    {
-
-      if (symbolP->sy_forward)
-	{
-	  S_SET_VALUE (symbolP, (S_GET_VALUE (symbolP)
-				 + S_GET_VALUE (symbolP->sy_forward)
-			       + symbolP->sy_forward->sy_frag->fr_address));
-
-	  if (
-#ifndef TE_I386AIX
-	       SF_GET_GET_SEGMENT (symbolP)
-#else
-	       SF_GET_GET_SEGMENT (symbolP)
-	       && S_GET_SEGMENT (symbolP) == SEG_UNKNOWN
-#endif /* TE_I386AIX */
-	    )
-	    {
-	      S_SET_SEGMENT (symbolP, S_GET_SEGMENT (symbolP->sy_forward));
-	    }			/* forward segment also */
-
-	  symbolP->sy_forward = 0;
-	}			/* if it has a forward reference */
-    }				/* walk the symbol chain */
-
   tc_crawl_symbol_chain (headers);
 
   /* The symbol list should be ordered according to the following sequence
@@ -1856,7 +1829,7 @@ obj_crawl_symbol_chain (headers)
 	      S_SET_SEGMENT (symbolP, SEG_TEXT);
 	    }			/* push data into text */
 
-	  S_SET_VALUE (symbolP, S_GET_VALUE (symbolP) + symbolP->sy_frag->fr_address);
+	  resolve_symbol_value (symbolP);
 
 	  if (!S_IS_DEFINED (symbolP) && !SF_GET_LOCAL (symbolP))
 	    {
@@ -2268,89 +2241,6 @@ obj_pre_write_hook (headers)
 		    section_alignment[(int) SEG_BSS]);
 }
 
-/* This is a copy from aout.  All I do is neglect to actually build the symbol. */
-
-static void
-obj_coff_stab (what)
-     int what;
-{
-  char *string;
-  expressionS e;
-  int goof = 0;			/* TRUE if we have aborted. */
-  int length;
-  int saved_type = 0;
-  long longint;
-  symbolS *symbolP = 0;
-
-  if (what == 's')
-    {
-      string = demand_copy_C_string (&length);
-      SKIP_WHITESPACE ();
-
-      if (*input_line_pointer == ',')
-	{
-	  input_line_pointer++;
-	}
-      else
-	{
-	  as_bad ("I need a comma after symbol's name");
-	  goof = 1;
-	}			/* better be a comma */
-    }				/* skip the string */
-
-  /*
-	 * Input_line_pointer->after ','.  String->symbol name.
-	 */
-  if (!goof)
-    {
-      if (get_absolute_expression_and_terminator (&longint) != ',')
-	{
-	  as_bad ("I want a comma after the n_type expression");
-	  goof = 1;
-	  input_line_pointer--;	/* Backup over a non-',' char. */
-	}			/* on error */
-    }				/* no error */
-
-  if (!goof)
-    {
-      if (get_absolute_expression_and_terminator (&longint) != ',')
-	{
-	  as_bad ("I want a comma after the n_other expression");
-	  goof = 1;
-	  input_line_pointer--;	/* Backup over a non-',' char. */
-	}			/* on error */
-    }				/* no error */
-
-  if (!goof)
-    {
-      get_absolute_expression ();
-
-      if (what == 's' || what == 'n')
-	{
-	  if (*input_line_pointer != ',')
-	    {
-	      as_bad ("I want a comma after the n_desc expression");
-	      goof = 1;
-	    }
-	  else
-	    {
-	      input_line_pointer++;
-	    }			/* on goof */
-	}			/* not stabd */
-    }				/* no error */
-
-  expression (&e);
-
-  if (goof)
-    {
-      ignore_rest_of_line ();
-    }
-  else
-    {
-      demand_empty_rest_of_line ();
-    }				/* on error */
-}				/* obj_coff_stab() */
-
 #ifdef BFD_ASSEMBLER
 static
 unsigned long
