@@ -61,13 +61,22 @@
 
 #ifdef TC_M68K
 #include "coff/m68k.h"
+#ifndef TARGET_FORMAT
 #define TARGET_FORMAT "coff-m68k"
 #endif
+#endif
+
+#ifdef TC_M88K
+#include "coff/m88k.h"
+#define TARGET_FORMAT "coff-m88kbcs"
+#endif
 
 #ifdef TC_I386
 #include "coff/i386.h"
+#ifndef TARGET_FORMAT
 #define TARGET_FORMAT "coff-i386"
 #endif
+#endif
 
 #ifdef TC_A29K
 #include "coff/a29k.h"
@@ -152,21 +161,24 @@ obj_symbol_type;
    section == 0 and value > 0 (external bss symbol) */
 #define S_IS_DEFINED(s)         ((s)->sy_symbol.ost_entry.n_scnum > C_UNDEF_SECTION || \
 				 ((s)->sy_symbol.ost_entry.n_scnum == C_UNDEF_SECTION && \
-				  (s)->sy_symbol.ost_entry.n_value > 0))
+				  S_GET_VALUE (s) > 0))
 /* True if a debug special symbol entry */
 #define S_IS_DEBUG(s)		((s)->sy_symbol.ost_entry.n_scnum == C_DEBUG_SECTION)
 /* True if a symbol is local symbol name */
-/* A symbol name whose name begin with ^A is a gas internal pseudo symbol */
-#define S_IS_LOCAL(s)		(S_GET_NAME(s)[0] == '\001' || \
-				 (s)->sy_symbol.ost_entry.n_scnum == C_REGISTER_SECTION || \
-				 (S_LOCAL_NAME(s) && !flagseen['L']))
+/* A symbol name whose name includes ^A is a gas internal pseudo symbol */
+#define S_IS_LOCAL(s) \
+  ((s)->sy_symbol.ost_entry.n_scnum == C_REGISTER_SECTION \
+   || (S_LOCAL_NAME(s) && !flagseen['L']) \
+   || (strchr (S_GET_NAME (s), '\001') != NULL))
 /* True if a symbol is not defined in this file */
-#define S_IS_EXTERN(s)		((s)->sy_symbol.ost_entry.n_scnum == 0 && (s)->sy_symbol.ost_entry.n_value == 0)
+#define S_IS_EXTERN(s)		((s)->sy_symbol.ost_entry.n_scnum == 0 \
+				 && S_GET_VALUE (s) == 0)
 /*
  * True if a symbol can be multiply defined (bss symbols have this def
  * though it is bad practice)
  */
-#define S_IS_COMMON(s)		((s)->sy_symbol.ost_entry.n_scnum == 0 && (s)->sy_symbol.ost_entry.n_value != 0)
+#define S_IS_COMMON(s)		((s)->sy_symbol.ost_entry.n_scnum == 0 \
+				 && S_GET_VALUE (s) != 0)
 /* True if a symbol name is in the string table, i.e. its length is > 8. */
 #define S_IS_STRING(s)		(strlen(S_GET_NAME(s)) > 8 ? 1 : 0)
 
@@ -177,8 +189,6 @@ obj_symbol_type;
 #define S_GET_OFFSET(s)         ((s)->sy_symbol.ost_entry.n_offset)
 /* The zeroes if symbol name is longer than 8 chars */
 #define S_GET_ZEROES(s)		((s)->sy_symbol.ost_entry.n_zeroes)
-/* The value of the symbol */
-#define S_GET_VALUE(s)		((unsigned) ((s)->sy_symbol.ost_entry.n_value))
 /* The numeric value of the segment */
 #define S_GET_SEGMENT(s)   s_get_segment(s)
 /* The data type */
@@ -195,8 +205,6 @@ obj_symbol_type;
 #define S_SET_OFFSET(s,v)	((s)->sy_symbol.ost_entry.n_offset = (v))
 /* The zeroes if symbol name is longer than 8 chars */
 #define S_SET_ZEROES(s,v)		((s)->sy_symbol.ost_entry.n_zeroes = (v))
-/* Set the value of the symbol */
-#define S_SET_VALUE(s,v)	((s)->sy_symbol.ost_entry.n_value = (v))
 /* The numeric value of the segment */
 #define S_SET_SEGMENT(s,v)	((s)->sy_symbol.ost_entry.n_scnum = SEGMENT_TO_SYMBOL_TYPE(v))
 /* The data type */
@@ -478,30 +486,30 @@ typedef struct
 
 stack;
 
+#define obj_segment_name(i) (segment_info[(int) (i)].scnhdr.s_name)
 
+#define obj_add_segment(s) obj_coff_add_segment (s)
 
-char *EXFUN (stack_pop, (stack * st));
-char *EXFUN (stack_push, (stack * st, char *element));
-char *EXFUN (stack_top, (stack * st));
-stack *EXFUN (stack_init, (unsigned long chunk_size, unsigned long element_size));
-void EXFUN (c_dot_file_symbol, (char *filename));
-void EXFUN (obj_extra_stuff, (object_headers * headers));
-void EXFUN (stack_delete, (stack * st));
+extern segT obj_coff_add_segment PARAMS ((const char *));
 
-segT EXFUN (s_get_segment,(struct symbol * ptr));
+extern void obj_coff_section PARAMS ((int));
 
-void EXFUN (c_section_header, (
+extern void c_dot_file_symbol PARAMS ((char *filename));
+extern void obj_extra_stuff PARAMS ((object_headers * headers));
+extern void stack_delete PARAMS ((stack * st));
 
-				struct internal_scnhdr * header,
-				char *name,
-				long core_address,
-				long size,
-				long data_ptr,
-				long reloc_ptr,
-				long lineno_ptr,
-				long reloc_number,
-				long lineno_number,
-				long alignment));
+extern segT s_get_segment PARAMS ((struct symbol * ptr));
+
+extern void c_section_header PARAMS ((struct internal_scnhdr * header,
+				      char *name,
+				      long core_address,
+				      long size,
+				      long data_ptr,
+				      long reloc_ptr,
+				      long lineno_ptr,
+				      long reloc_number,
+				      long lineno_number,
+				      long alignment));
 
 
 /* sanity check */
@@ -518,6 +526,22 @@ extern struct internal_scnhdr text_section_header;
 extern SCNHDR data_section_header;
 extern SCNHDR text_section_header;
 #endif
-#endif
 
-/* end of obj-coffbfd.h */
+/* Forward the segment of a forwarded symbol.  */
+#define obj_frob_forward_symbol(symp) \
+  (SF_GET_GET_SEGMENT (symp) \
+   ? (S_SET_SEGMENT (symp, S_GET_SEGMENT (symp->sy_value.X_add_symbol)), 0) \
+   : 0)
+
+/* Stabs in a coff file go into their own section. */
+
+#define SEPARATE_STAB_SECTIONS
+
+/* We need 12 bytes at the start of the section to hold some initial
+   information.  */
+
+extern void obj_coff_init_stab_section PARAMS ((segT));
+
+#define INIT_STAB_SECTION(seg) obj_coff_init_stab_section (seg)
+
+#endif /* OBJ_FORMAT_H */
